@@ -191,10 +191,10 @@ class TupleDataLoader(TupleActionProcessorDelegateABC, TuplesProviderABC):
         results = [result]
 
         payloadEnvelope = PayloadEnvelope(filt=filt)
-        payloadEnvelope.encodedPayload = Payload(
+        payloadEnvelope.encodedPayload = yield Payload(
             filt=filt, tuples=results
-        ).toEncodedPayload()
-        vortexMsg = payloadEnvelope.toVortexMsg()
+        ).toEncodedPayloadDefer()
+        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
 
         return vortexMsg
 
